==> redux-0.3.0.tgz: package/lib/createDispatcher.js <==
'use strict';

var _Object$defineProperty = require('babel-runtime/core-js/object/define-property')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

_Object$defineProperty(exports, '__esModule', {
  value: true
});

exports['default'] = createDispatcher;

var _lodashObjectMapValues = require('lodash/object/mapValues');

var _lodashObjectMapValues2 = _interopRequireDefault(_lodashObjectMapValues);

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

// An action dispatched to init store state
var BOOTSTRAP_STORE = {
  type: 'BOOTSTRAP_STORE'
};

function createDispatcher() {
  var observers = {};
  var stores = {};
  var currentState = {};

  // To compute the next state, combine the next states of every store
  function computeNextState(state, action) {
    return (0, _lodashObjectMapValues2['default'])(stores, function (store, key) {
      var nextStoreState = store(state[key], action);
      (0, _invariant2['default'])(nextStoreState != null, 'State returned by %s is null or undefined.', key);
      return nextStoreState;
    });
  }

  // Notify observers about the changed stores
  function emitChange(changedKeys) {
    if (!changedKeys.length) {
      return;
    }

    // Gather the affected observers
    var notifyObservers = [];
    changedKeys.forEach(function (key) {
      observers[key].forEach(function (o) {
        if (notifyObservers.indexOf(o) === -1) {
          notifyObservers.push(o);
        }
      });
    });

    // Emit change
    notifyObservers.forEach(function (o) {
      return o();
    });
  }

  // Update state and emit change if needed
  function updateState(nextState) {
    // Swap the state
    var previousState = currentState;
    currentState = nextState;

    // Notify the observers
    var changedKeys = _Object$keys(currentState).filter(function (key) {
      return currentState[key] !== previousState[key];
    });
    emitChange(changedKeys);
  }

  // Reassign the current state on each dispatch
  function dispatch(action) {
    (0, _invariant2['default'])(typeof action.type === 'string', 'Action type must be a string.');

    var nextState = computeNextState(currentState, action);
    updateState(nextState);
  }

  // Merge the newly added stores
  function mergeStores(newStores) {
    newStores.forEach(function (newStore) {
      var key = newStore.name;
      stores[key] = newStore;
      observers[key] = observers[key] || [];
    });
    dispatch(BOOTSTRAP_STORE);
  }

  // Provide subscription and unsubscription
  function observeStores(observedStores, onChange) {
    mergeStores(observedStores);
    var observedKeys = observedStores.map(function (s) {
      return s.name;
    });

    // Emit the state update
    function handleChange() {
      onChange(currentState);
    }

    // Synchronously emit the initial value
    handleChange();

    // Register the observer for each relevant key
    observedKeys.forEach(function (key) {
      return observers[key].push(handleChange);
    });

    // Let it unregister when the time comes
    return function () {
      observedKeys.forEach(function (key) {
        var index = observers[key].indexOf(handleChange);
        observers[key].splice(index, 1);
      });
    };
  }

  // Bind an action creator to the dispatcher
  function wrapActionCreator(actionCreator) {
    return function dispatchAction() {
      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      var action = actionCreator.apply(undefined, args);
      if (typeof action === 'function') {
        // Callback-style action creator
        action(dispatch, currentState);
      } else {
        // Simple action creator
        dispatch(action);
      }
    };
  }

  return {
    wrapActionCreator: wrapActionCreator,
    observeStores: observeStores
  };
}

module.exports = exports['default'];

==> redux-0.5.0.tgz: package/lib/createDispatcher.js <==
'use strict';

var _Object$defineProperty = require('babel-runtime/core-js/object/define-property')['default'];

var _Map = require('babel-runtime/core-js/map')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

_Object$defineProperty(exports, '__esModule', {
  value: true
});

exports['default'] = createDispatcher;

var _lodashObjectMapValues = require('lodash/object/mapValues');

var _lodashObjectMapValues2 = _interopRequireDefault(_lodashObjectMapValues);

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

// An action dispatched to init store state
var BOOTSTRAP_STORE = {
  type: 'BOOTSTRAP_STORE'
};

function createDispatcher() {
  var observers = {};
  var stores = {};
  var storeKeys = new _Map();
  var currentState = {};

  // To compute the next state, combine the next states of every store
  function computeNextState(state, action) {
    return (0, _lodashObjectMapValues2['default'])(stores, function (store, key) {
      var nextStoreState = store(state[key], action);
      (0, _invariant2['default'])(nextStoreState != null, 'State returned by %s is null or undefined.', key);
      return nextStoreState;
    });
  }

  // Notify observers about the changed stores
  function emitChange(changedKeys) {
    if (!changedKeys.length) {
      return;
    }

    // Gather the affected observers
    var notifyObservers = [];
    changedKeys.forEach(function (key) {
      observers[key].forEach(function (o) {
        if (notifyObservers.indexOf(o) === -1) {
          notifyObservers.push(o);
        }
      });
    });

    // Emit change
    notifyObservers.forEach(function (o) {
      return o();
    });
  }

  // Update state and emit change if needed
  function updateState(nextState) {
    // Swap the state
    var previousState = currentState;
    currentState = nextState;

    // Notify the observers
    var changedKeys = _Object$keys(currentState).filter(function (key) {
      return currentState[key] !== previousState[key];
    });
    emitChange(changedKeys);
  }

  // Reassign the current state on each dispatch
  function dispatch(action) {
    (0, _invariant2['default'])(typeof action.type === 'string', 'Action type must be a string.');

    var nextState = computeNextState(currentState, action);
    updateState(nextState);
  }

  // Merge the newly added stores
  function receiveStores(nextStores) {
    _Object$keys(nextStores).forEach(function (key) {
      stores[key] = nextStores[key];
      observers[key] = observers[key] || [];
      storeKeys[stores[key]] = key;
    });
    dispatch(BOOTSTRAP_STORE);
  }

  // Provide subscription and unsubscription
  function observeStores(observedStores, onChange) {
    var observedKeys = observedStores.map(function (store) {
      var key = storeKeys[store];
      (0, _invariant2['default'])(key, 'This store is not registered with the Redux root: %s', store);
      return key;
    });

    // Emit the state update
    function handleChange() {
      onChange(currentState);
    }

    // Synchronously emit the initial value
    handleChange();

    // Register the observer for each relevant key
    observedKeys.forEach(function (key) {
      return observers[key].push(handleChange);
    });

    // Let it unregister when the time comes
    return function () {
      observedKeys.forEach(function (key) {
        var index = observers[key].indexOf(handleChange);
        observers[key].splice(index, 1);
      });
    };
  }

  // Bind an action creator to the dispatcher
  function wrapActionCreator(actionCreator) {
    return function dispatchAction() {
      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      var action = actionCreator.apply(undefined, args);
      if (typeof action === 'function') {
        // Callback-style action creator
        action(dispatch, currentState);
      } else {
        // Simple action creator
        dispatch(action);
      }
    };
  }

  return {
    wrapActionCreator: wrapActionCreator,
    observeStores: observeStores,
    receiveStores: receiveStores
  };
}

module.exports = exports['default'];
